import {Popup, ProjectModel, Toast, Toolbar} from "@bryntum/gantt";
import {GanttViewStateModel} from "src/main/typescript/components/BGantt/GanttViewStateModel";
import {
    cedTaskGroup,
    collapseGroup,
    planList,
    settingsMenu,
    syncGroup,
    viewMenu,
    viewSelector,
    zoomGroup
} from "src/main/typescript/components/BGantt/BGToolbar/BGToolbarElements";
import {TaskExtModel} from "src/main/typescript/components/BGantt/TaskExtModel";
import {TaskExtStore} from "src/main/typescript/components/BGantt/store/TaskExtStore";
import {KeyResultStore} from "src/main/typescript/components/BGantt/store/KeyResultStore";
import DraftWorkPlanSynchronizer
    from "src/main/typescript/components/BGantt/plansynchronizer/DraftWorkPlanSynchronizer";


/**
 * DEPRECATED
 */
export default class BGToolbar extends Toolbar {


    constructor(config) {
        super(config);
    }

    // Factoryable type name
    static get type() {
        return 'gantttoolbar';
    }

    static get $name() {
        return 'GanttToolbar';
    }

    // Called when toolbar is added to the Gantt panel
    set parent(parent) {
        super.parent = parent;
        const me = this;
        me.gantt = parent;
        me.styleNode = document.createElement('style');
        document.head.appendChild(me.styleNode);
    }

    get parent() {
        return super.parent;
    }

    static get configurable() {
        return {
            items: [

                syncGroup,
                cedTaskGroup,
                collapseGroup,
                zoomGroup,
                planList,
                viewSelector,
                viewMenu,
                '->',
                settingsMenu
            ]
        };
    }


    onFullScreen() {
        if (this.gantt.fullScreen) {
            this.gantt.fullScreen = false
            this.toggleFullscreen(false)
        } else {
            this.gantt.fullScreen = true
            this.toggleFullscreen(true)
        }
    }

    onSaveButtonClick(event) {
        if (this.isReadOnlyStatus()) {
            return
        }
        this.gantt.project.sync()
    }


    onReloadButtonClick(event) {
        if (this.isReadOnlyStatus()) {
            return
        }
        this.gantt.project.load()
    }


    onLoadGanttStateFromFile(event) {
        console.error("LOAD FROM FILE GANTT's VIEW STATE FROM SERVER")

    }

    onSettingsClick({source: item}) {
        const {gantt} = this;
        if (item.subGrid) {
            const subGrid = gantt.subGrids[item.subGrid];
            subGrid.collapsed = !subGrid.collapsed;
        }
    }

    onSettingsShow({source: menu}) {
        const {gantt} = this;

        menu.items.map(item => {
            const {feature} = item;
            if (item.subGrid) {
                item.checked = gantt.subGrids[item.subGrid].collapsed;
            }
        });
    }

    onUISettingsShow({source: menu}) {
        const
            {gantt} = this,
            {rowHeight, barMargin} = menu.widgetMap;
        rowHeight.value = gantt.rowHeight;
        barMargin.value = gantt.barMargin;
        barMargin.max = (gantt.rowHeight / 2) - 5;
    }

    onFeaturesClick({source: item}) {
        const {gantt} = this;
        if (item.feature) {
            const feature = gantt.features[item.feature];
            gantt.features[item.feature].disabled = !feature.disabled;

        } else if (item.subGrid) {
            const subGrid = gantt.subGrids[item.subGrid];
            subGrid.collapsed = !subGrid.collapsed;
        }
    }

    onFeaturesShow({source: menu}) {
        const {gantt} = this;

        menu.items.map(item => {
            const {feature} = item;
            if (feature) {
                // a feature might be not presented in the gantt
                // (the code is shared between "advanced" and "php" demos which use a bit different set of features)
                if (gantt.features[feature]) {
                    item.checked = !gantt.features[feature].disabled;
                }
                // hide not existing features
                else {
                    item.hide();
                }
            } else if (item.subGrid) {
                item.checked = gantt.subGrids[item.subGrid].collapsed;
            }
        });
    }

    /**
     *  Check readOnly status
     * @returns {boolean} if readOnly = true, else false
     */
    isReadOnlyStatus() {
        if (!this.gantt.selectedPlan) {
            Toast.show('Select plan');
            return true
        }
        if (this.gantt.readOnly) {
            Toast.show('Selected plan has readOnly Status');
            return true
        }
        return false
    }

    async onAddTaskClick() {
        if (this.isReadOnlyStatus()) {
            return
        }
        const {gantt} = this

        let zeroTask = gantt.taskStore.getRootTask()
        const added = zeroTask.appendChild({
            name: gantt.L("New Task"),
            duration: 1
        })

        // run propagation to calculate new task fields
        await gantt.project.propagateAsync();

        // scroll to the added task
        await gantt.scrollRowIntoView(added);

        gantt.features.cellEdit.startEditing({
            record: added,
            field: 'name'
        });
    }


    onEditTaskClick() {
        const {gantt} = this;

        if (gantt.selectedRecord) {
            gantt.editTask(gantt.selectedRecord);
        } else {
            Toast.show(gantt.L('First select the task you want to edit'));
        }
    }

    onDeleteTaskClick() {
        const {gantt} = this;
        if (this.isReadOnlyStatus()) {
            return
        }
        if (gantt.selectedRecords.length !== 0) {
            gantt.taskStore.remove(gantt.selectedRecords)
        } else {
            Toast.show(gantt.L('First select the task you want to delete'));
        }
    }

    onExpandAllClick() {
        this.gantt.expandAll();
    }

    onCollapseAllClick() {
        this.gantt.collapseAll();
    }

    onZoomInClick() {
        this.gantt.zoomIn();
    }

    onZoomOutClick() {
        this.gantt.zoomOut();
    }

    onZoomToFitClick() {
        this.gantt.zoomToFit({
            leftMargin: 50,
            rightMargin: 50
        });
    }

    onRowHeightChange({value, source}) {
        this.gantt.rowHeight = value;
        source.owner.widgetMap.barMargin.max = (value / 2) - 5;
    }

    onBarMarginChange({value}) {
        this.gantt.barMargin = value;
    }

    /**
     * The function returns an object to be saved in the database containing information about the current state of the gantt display
     * The first call can be made after the event mountComponent when the bryntumgantt object is fully formed, i.e. in
     * didComponentMount and later
     * If we call the function before, the function will do nothing.
     */
    collectGanttState() {
        let gantt = this.gantt
        if (!gantt) {
            throw new Error('ERROR: GANTT NULLABLE. You may have called the function too early. Read the documentation for the function');
        }
        let ganttViewState = {}
        ganttViewState.state = {
            barMargin: gantt.state.barMargin,
            rowHeight: gantt.state.rowHeight,
            // zoomLevel: gantt.state.zoomLevel,
            // zoomLevelOptions: gantt.state.zoomLevelOptions,
        }
        ganttViewState.features = {
            dependencies: {disabled: gantt.features.dependencies.disabled},
            criticalPaths: {disabled: gantt.features.criticalPaths.disabled},
            labels: {disabled: gantt.features.labels.disabled}
        }

        //TODO костыль фильтрации сохранения timeAxis. Колонки, которая отвечает за отображение колбасок ганта
        ganttViewState.columns = gantt.columns.records.filter((it) => {
            return it.data.type !== "timeAxis"
        }).map((it) => {
            return {
                id: it.data.id,
                type: it.data.type,
                width: it.data.width,
                field: it.data.field,
                hidden: it.data.hidden,
                format: it.data.format
            }
        })

        return ganttViewState
    }


    onSyncRequestClick() {
        const {gantt} = this;
        if (gantt.project.crudStoreHasChanges()) {
            Toast.show(this.gantt.L("There are unsaved changes"));
            return
        }
        if (!gantt.project.currentWorkPlan) {
            Toast.show(this.gantt.L("Work plan not found"));
            return
        }
        this.gantt.masked = {text: this.gantt.L("Synchronization with work plan")}
        let sourceProjectModel = new ProjectModel({
            taskModelClass: TaskExtModel,
            autoLoad: false,
            autoSync: false,
            writeAllFields: true,
            taskStoreClass: TaskExtStore,
            crudStores: [new KeyResultStore()],
            transport: {
                load: {
                    url: '/api/v1/gantt/synchronizer/load',
                    params: {
                        projectId: gantt.projectId,
                        planId: gantt.project.currentWorkPlan
                    }
                }
            },
            validateResponse: true
        });
        sourceProjectModel.load().then(() => {
            let synchronizer = new DraftWorkPlanSynchronizer({
                sourceProject: sourceProjectModel,
                synchronizedProject: gantt.project
            })
            return synchronizer.synchronize()
        }).catch((info) => {
            console.error(info)
            gantt.project.revertChanges()
            Toast.show(this.gantt.L("Something went wrong"));
        }).finally(() => {
            this.gantt.masked = false
        })
    }

    createNewView(viewName, viewState) {
        this.createGanttViewState({
            variables: {
                name: viewName,
                accountingObjectId: this.gantt.projectId,
                state: viewState
            }
        }).then((it) => {
            if (it.data) {
                this.ganttViewStateStore.add(new GanttViewStateModel({
                    // @ts-ignore
                    name: it.data?.packet?.createGanttViewState.name,
                    id: it.data?.packet?.createGanttViewState.id
                }))
            }
        })
    }

    onAddGanttViewState() {
        if (this.isReadOnlyStatus()) {
            return
        }
        const popup = new Popup({
            header: 'Create View',
            forElement: this.gantt,
            autoShow: false,
            modal: true,
            centered: true,
            closable: true,
            closeAction: 'destroy',
            width: '20em',
            minHeight: '18em',
            bbar: [
                {
                    text: 'Cancel',
                    minWidth: 100,
                    onAction: 'up.close'
                },
                {
                    text: 'OK',
                    minWidth: 100,
                    cls: 'b-raised b-blue',
                    onAction: () => {
                        let viewName = popup.widgetMap.nameField.value
                        if (popup.widgetMap.fileField.files[0]) {
                            let reader = new FileReader();
                            reader.onload = (event) => {
                                this.createNewView(viewName, event.target.result)
                            }
                            reader.readAsText(popup.widgetMap.fileField.files[0])
                        } else {
                            let state = JSON.stringify(this.collectGanttState())
                            this.createNewView(viewName, state)
                        }
                        popup.close();
                    }
                }
            ],
            items: [
                {
                    ref: 'nameField',
                    type: 'text',
                    label: 'View Name'
                },
                {
                    ref: "fileField",
                    type: "filefield",
                    width: 300,
                    accept: '.json',
                    label: 'Optional: pick gantt state json',
                    style: 'display: block; margin-top: 2em;'
                }
            ]
        });
        window.createPopup = popup
        popup.show();
    }

    inGanttViewSelected() {
        if (!this.gantt.selectedViewId) {
            console.error("ERROR: View not selected")
            Toast.show("First choice view");
            return false
        }
        return true
    }

    onDeleteGanttViewState() {
        if (!this.inGanttViewSelected()) {
            return
        }
        this.deleteGanttViewState({variables: {ganttViewStateId: this.gantt.selectedViewId}}).then((it) => {
            if (!it.error) {
                this.ganttViewStateStore.remove(this.gantt.selectedViewId)
                this.setSelectedViewId("")
            }
        })
    }

    onUpdateGanttViewState(event) {
        if (!this.inGanttViewSelected()) {
            return
        }
        this.updateGanttViewState({
            variables: {
                ganttViewStateId: this.gantt.selectedViewId,
                state: JSON.stringify(this.collectGanttState())
            }
        })
    }

    onSelectGanttViewStata(event) {
        this.setSelectedViewId(event.value)
    }

    onExportGanttStateToFile(event) {
        const filename = 'ganttView.json';
        const jsonStr = JSON.stringify(this.collectGanttState());

        let element = document.createElement('a');
        element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(jsonStr));
        element.setAttribute('download', filename);

        element.style.display = 'none';
        document.body.appendChild(element);

        element.click();

        document.body.removeChild(element);
    }
}

// Register this widget type with its Factory
//BGToolbar.initClass();
