export const saveOnServerElement = {
    color: 'b-blue',
    icon: 'b-fa b-fa-save',
    tooltip: 'L{Save changes on the server}',
    onAction: 'up.onSaveButtonClick'
}

export const reloadFromServerElement = {
    color: 'b-red',
    icon: 'b-fa b-fa-sync-alt',
    tooltip: 'L{Reload data from the server}',
    onAction: 'up.onReloadButtonClick',
    disabled: true
}

export const syncGroup = {
    type: 'buttonGroup',
    items: [
        {
            ...saveOnServerElement,
            ref: "saveOnServerElementToolbar"
        },
        {
            ...reloadFromServerElement,
            ref: "reloadFromServerElementToolbar"
        }
    ]
}

export const cedTaskGroup = {
    type: 'buttonGroup',
    items: [
        {
            color: 'b-green',
            ref: 'addTaskButton',
            icon: 'b-fa b-fa-plus',
            text: 'L{Task}',
            tooltip: 'L{Create new task}',
            onAction: 'up.onAddTaskClick'
        },
        {
            ref: 'editTaskButton',
            icon: 'b-fa b-fa-pen',
            text: 'L{Edit}',
            tooltip: 'L{Edit selected task}',
            onAction: 'up.onEditTaskClick'
        },
        {
            color: 'b-red',
            ref: 'deleteTaskButton',
            icon: 'b-fa b-fa-minus',
            text: 'L{Delete}',
            tooltip: 'L{Delete selected tasks}',
            onAction: 'up.onDeleteTaskClick',
        },
        // {
        //     //TODO Временно отключено. Тикет RNDTEAM-56
        //     ref: 'undoRedo',
        //     type: 'undoredo',
        //     items: {
        //         transactionsCombo: null
        //     },
        //     disabled : true
        // }
    ]
}

export const collapseGroup = {
    type: 'buttonGroup',
    items: [
        {
            ref: 'expandAllButton',
            icon: 'b-fa b-fa-angle-double-down',
            tooltip: 'L{Expand all}',
            onAction: 'up.onExpandAllClick'
        },
        {
            ref: 'collapseAllButton',
            icon: 'b-fa b-fa-angle-double-up',
            tooltip: 'L{Collapse all}',
            onAction: 'up.onCollapseAllClick'
        }
    ]
}

export const zoomGroup = {
    type: 'buttonGroup',
    items: [
        {
            ref: 'zoomInButton',
            icon: 'b-fa b-fa-search-plus',
            tooltip: 'L{Zoom in}',
            onAction: 'up.onZoomInClick'
        },
        {
            ref: 'zoomOutButton',
            icon: 'b-fa b-fa-search-minus',
            tooltip: 'L{Zoom out}',
            onAction: 'up.onZoomOutClick'
        },
        {
            ref: 'zoomToFitButton',
            icon: 'b-fa b-fa-compress-arrows-alt',
            tooltip: 'L{Zoom to fit}',
            onAction: 'up.onZoomToFitClick'
        }
    ]
}

export const planList = {
    type: "button",
    ref: 'planPageButton',
    icon: 'b-fa b-fa-link',
    text: 'L{PlanList}',
    tooltip: 'L{Go to plan list page}',
    onAction: 'up.onPlanPage'
}
export const syncRequest = {
    ref: 'syncRequestButton',
    icon: 'b-fa b-fa-link',
    text: 'L{Synchronization}',
    tooltip: 'L{Synchronize with work plan}',
    onItem: 'up.onSyncRequestClick'
}


// export const fullScreenButton = {
//     type: "button",
//     ref: 'fullscreenButton',
//     icon: 'b-fa b-fa-expand',
//     tooltip: 'Go to plan list page',
//     onAction: 'up.onFullScreen'
// }

export const viewSelector = {
    type: "combo",
    id: "ganttViewStateCombo",
    valueField: 'id',
    store: 'up.ganttViewStateStore',
    displayField: 'name',
    onAction: "up.onSelectGanttViewStata",
    ref: 'ganttViewStateCombo',
    style: {marginRight: '.5em'}
}

export const viewMenu = {
    type: "button",
    id: "viewMenuButton",
    ref: 'viewMenuButton',
    icon: 'b-fa-sliders-h',
    text: 'L{View}',
    tooltip: 'L{Gantt View actions}',
    toggleable: true,
    menu: {
        type: 'menu',
        anchor: true,
        cls: 'settings-menu',
        layoutStyle: {
            flexDirection: 'column'
        },
        items: [
            {
                color: 'b-green',
                ref: 'addGanttViewStateButton',
                icon: 'b-fa b-fa-plus',
                text: 'L{View}',
                tooltip: 'L{Create new gantt view state}',
                onItem: 'up.onAddGanttViewState'
            },
            {
                ref: 'saveGanttStateButton',
                icon: 'b-fa b-fa-cloud-upload-alt',
                text: 'L{Save to server}',
                tooltip: 'L{Save view state to server}',
                onItem: 'up.onUpdateGanttViewState'
            },
            {
                color: 'b-red',
                ref: 'deleteGanttStateButton',
                icon: 'b-fa b-fa-trash-alt',
                text: 'L{Delete}',
                tooltip: 'L{Delete view state}',
                onItem: 'up.onDeleteGanttViewState'
            },
            {
                ref: 'exportGanttStateToFileButton',
                icon: 'b-fa b-fa-file-export',
                text: 'L{Export to file}',
                tooltip: 'L{Export current view state to json}',
                onItem: 'up.onExportGanttStateToFile',
            }
        ]
    }
}

export const settingsMenu = {
    type: "button",
    id: "settingsButton",
    ref: 'settingsButton',
    icon: 'b-fa b-fa-tasks',
    text: 'L{Settings}',
    tooltip: 'L{Gantt scene settings}',
    toggleable: true,
    menu: {
        onItem: 'up.onSettingsClick',
        onBeforeShow: 'up.onSettingsShow',
        items: [{
            text: 'L{UI}',
            ref: 'uiSettings',
            icon: 'b-fa-sliders-h',
            menu: {
                type: 'popup',
                anchor: true,
                cls: 'settings-menu',
                layoutStyle: {
                    flexDirection: 'column'
                },
                onBeforeShow: 'up.onUISettingsShow',

                items: [
                    {
                        type: 'slider',
                        ref: 'rowHeight',
                        text: 'L{Row height}',
                        width: '12em',
                        showValue: true,
                        min: 20,
                        max: 70,
                        onInput: 'up.onRowHeightChange'
                    },
                    {
                        type: 'slider',
                        ref: 'barMargin',
                        text: 'L{Bar margin}',
                        width: '12em',
                        showValue: true,
                        min: 0,
                        max: 10,
                        onInput: 'up.onBarMarginChange'
                    }
                ]
            }
        }, {
            text: 'L{Features}',
            ref: 'featuresSettings',
            icon: 'b-fa-sliders-h',
            menu: {
                type: 'menu',
                anchor: true,
                cls: 'settings-menu',
                layoutStyle: {
                    flexDirection: 'column'
                },
                onItem: 'up.onFeaturesClick',
                onBeforeShow: 'up.onFeaturesShow',
                items: [
                    {
                        text: 'L{Draw dependencies}',
                        ref: 'toggleDependencies',
                        feature: 'dependencies',
                        tooltip: 'L{Show/Side dependencies arrow}',
                        checked: false
                    },
                    {
                        text: 'L{Task labels}',
                        tooltip: 'L{Show/Side task labels}',
                        reg: 'toggleLabels',
                        feature: 'labels',
                        checked: false
                    },
                    {
                        text: 'Critical paths',
                        ref: 'toggleCriticalPaths',
                        feature: 'criticalPaths',
                        tooltip: 'Show/Side critical paths',
                        checked: false
                    }
                ]
            }
        }, syncRequest
        ]
    }


}