import {TaskExtModel} from "src/main/typescript/components/BGantt/TaskExtModel";
import {AssignmentModelExt} from "src/main/typescript/components/BGantt/model/AssignmentModelExt";
import {TaskExtStore} from "src/main/typescript/components/BGantt/store/TaskExtStore";
import {AssignmentExtStore} from "src/main/typescript/components/BGantt/store/AssignmentExtStore";
import {KeyResultStore} from "src/main/typescript/components/BGantt/store/KeyResultStore";
import {ProjectModel, ProjectModelConfig, Store} from "@bryntum/gantt";
import {TaskCostCategoryStore} from "src/main/typescript/components/BGantt/store/TaskCostCategoryStore";
import {getKeyCloakInstance} from "src/main/typescript/Utils/hoc/withAuthApolloClient";
import {AccObjectInfo} from "src/main/typescript/components/BGantt/toolbar/GToolBar";

interface ProjectModelExtConfig extends ProjectModelConfig {
    projectId: String
    selectedPlan: String
    keyResultStore: Store
    taskCostCategoryStore: Store

}

export const getProjectModelConfig = (accountingObjectId: string, planId: string, authorizationToken?: string): Partial<ProjectModelExtConfig> => {
    let keyResultStore = new KeyResultStore()
    let taskCostCategoryStore = new TaskCostCategoryStore()
    let configuration = {
        taskModelClass: TaskExtModel,
        assignmentModelClass: AssignmentModelExt,
        autoLoad: false,
        autoSync: false,
        writeAllFields: true,
        taskStoreClass: TaskExtStore,
        calendarManagerStore: {tree: true, transformFlatData: true},
        assignmentStoreClass: AssignmentExtStore,
        crudStores: [keyResultStore, taskCostCategoryStore],
        keyResultStore: keyResultStore,
        taskCostCategoryStore: taskCostCategoryStore,
        hoursPerDay: 8,
        transport: {
            load: {
                url: '/api/v1/gantt/synchronizer/load',
                params: {projectId: accountingObjectId, planId: planId}
            },
            sync: {
                url: '/api/v1/gantt/synchronizer/sync',
                params: {projectId: accountingObjectId, planId: planId}
            }
        },
        validateResponse: true
    }
    if (authorizationToken) {
        configuration.transport.load.headers = {
            Authorization: authorizationToken
        }
        configuration.transport.sync.headers = {
            Authorization: authorizationToken
        }
    }
    return configuration
}

export const useProjectModelConfigWithAuth = (accountingObjectId: string, planId: string): Partial<ProjectModelExtConfig> => {
    const keycloak = getKeyCloakInstance();
    return getProjectModelConfig(accountingObjectId, planId, "Bearer " + keycloak.token)
}
/**
 * 1. Создаем план через graphql
 * 2. Создаем сущность
 * 3. Загружаем данные с сервера projectModel.load()
 * 4. Меняем данные в хранилище
 * 5. Коммитим await projectModel.commitAsync()
 * 6. Сохраняем изменения на сервер projectModel.sync()
 * Документация
 * https://www.bryntum.com/docs/gantt/api/Gantt/model/ProjectModel
 * Примеры можно смотреть в AbstractPlanSynchronizer
 * А также очень подробная документация у ганта
 *
 * Операции с хранилищем можно делать батчами
 * смотрим beginBatch/endBatch
 *
 *
 * @param accountingObjectId
 * @param planId
 */
export const createProjectModel = (accountingObjectId: String, planId: String): ProjectModel => {
    let config = getProjectModelConfig(accountingObjectId, planId)
    return new ProjectModel(config)
}