import {GanttFeaturesConfigType, Model} from "@bryntum/gantt";
import {Maybe} from "@graphql-tools/utils";
import {Column as ColumnStoreColumn} from "@bryntum/gantt";

/**
 * Column gantt type. Used to save the display state of gantt. Used to create columns via ColumnStore
 */
export interface Column extends ColumnStoreColumn{
    id: string,
    type: string,
    field?: string
}

/**
 * Gantt display state
 */
interface GanttStateType {
    barMargin?: number,
    rowHeight?: number,
    zoomLevel?: number
    zoomLevelOptions?: {
        centerDate: Date,
        endDate: Date,
        startDate: Date,
        width: number
    }
}

export interface GanttViewStateType {
    type: "GanttViewStateType",
    features?: Partial<GanttFeaturesConfigType>,
    filters?: Maybe<object|object[]>
    sorters?: Maybe<object|object[]>
    state?: GanttStateType,
    columns?: Array<Column>
}



/**
 * This is used to display the View list.
 * Currently saturated through graphql, with only one name field. All other information is obtained by "request" from the graphql server.
 * If necessary, you can saturate this other information and use it through the gantt tools
 */
export class GanttViewStateModel extends Model {
    static get fields() {
        return [
            {name: "name", type: "string"},
            // {name: "features", type: "object"},
            // {name: "state", type: "object"},
            // {name: "columns", type: "object"},
        ];
    }
}