import {Model, ModelConfig, TaskModel} from "@bryntum/gantt";

export class TaskExtModel extends TaskModel {
    static get fields() {
        return [
            {name: "critical", type: "boolean", persist: true, alwaysWrite: true},
            {name: "earlyStartDate", type: "date", persist: true, alwaysWrite: true},
            {name: "earlyEndDate", type: "date", persist: true, alwaysWrite: true},
            {name: "lateStartDate", type: "date", persist: true, alwaysWrite: true},
            {name: "lateEndDate", type: "date", persist: true, alwaysWrite: true},
            {name: "keyMilestone", type: "boolean"},
            {name: "keyResult", type: "string"},
            {name: "taskCostCategory", type: "string"},
            {name: "guid", type: "string"},
            {name: "rootTask", type: "boolean"},
            {name: "hyperlink", type: "string"},
            {name: "hyperlinkAddress", type: "string"},
            {name: "taskKind", type: "object"},
        ];
    }

    set hyperlink(value) {
        super.set("hyperlink", value)
        if (!this.hyperlinkAddress) {
            this.hyperlinkAddress = value
        }
    }

    set hyperlinkAddress(value) {
        if (!this.hyperlink || this.hyperlink == this.hyperlinkAddress) {
            super.set("hyperlink", value)
        }
        super.set("hyperlinkAddress", value)
    }

    get hyperlink() {
        // @ts-ignore
        return super.data["hyperlink"]
    }

    get hyperlinkAddress() {
        // @ts-ignore
        return super.data["hyperlinkAddress"]
    }

    taskCostCategoryIsChangeable(value: string | undefined | null) {
        let parent = this.parent as TaskExtModel
        return !(parent && parent.taskCostCategory && parent.taskCostCategory != value)
    }

    set taskCostCategory(value) {
        super.set("taskCostCategory", value)
        if (!this.taskCostCategoryIsChangeable(value)) {
            let message = "You cannot override taskCostCategory in a child task"
            console.error(message)
            return
        }
        if (this.children && Array.isArray(this.children)) {
            if (this.taskStore) {
                this.taskStore.beginBatch()
            }
            this.children.forEach((child: TaskExtModel) => {
                child.taskCostCategory = value
            })
            if (this.taskStore) {
                this.commitAsync().then(() => {
                    this.taskStore.endBatch()
                })
            }
        }

    }

    get taskCostCategory() {
        //@ts-ignore
        return super.data["taskCostCategory"]
    }


    // @ts-ignore
    copy(...args) {
        const copy = super.copy(...args);
        // @ts-ignore
        copy.setData('guid', null)
        return copy;

    }

    inheritTaskCostCategory(childRecord: Model | Model[] | object | object[] | Partial<ModelConfig> | Partial<ModelConfig>[]) {
        if (childRecord) {
            if (Array.isArray(childRecord)) {
                childRecord.forEach((task: TaskExtModel) => {
                    if (this.taskCostCategory && task.taskCostCategory != this.taskCostCategory) {
                        task.taskCostCategory = this.taskCostCategory
                    }
                })
            } else {
                // @ts-ignore
                if (this.taskCostCategory && childRecord.taskCostCategory != this.taskCostCategory) {
                    // @ts-ignore
                    childRecord.taskCostCategory = this.taskCostCategory
                }
            }
        }
    }

    appendChild(childRecord: Model | Model[] | object | object[] | Partial<ModelConfig> | Partial<ModelConfig>[], silent?: boolean): Model | Model[] | null {
        this.inheritTaskCostCategory(childRecord)
        return super.appendChild(childRecord, silent);
    }

    insertChild(childRecord: Model | Model[] | object | object[] | Partial<ModelConfig> | Partial<ModelConfig>[], before?: Model, silent?: boolean): Model | Model[] | null {
        this.inheritTaskCostCategory(childRecord)
        return super.insertChild(childRecord, before, silent);
    }

    constructor(data?: any, store?: any, meta?: any) {
        // @ts-ignore
        super(data, store, meta);
        if (!data.taskCostCategory && this.parent) {
            // @ts-ignore
            this.taskCostCategory = this.parent.taskCostCategory
        }
    }
}