import {Column, ColumnStore} from "@bryntum/gantt";
import KeyResultField from "../widget/KeyResultField";

export default class KeyResultColumn extends Column {
    static get $name() {
        return 'KeyResultColumn';
    }

    // unique alias of the column
    static get type() {
        return 'keyResult';
    }

    // indicates that the column should be present in "Add New..." column
    static get isGanttColumn() {
        return true;
    }

    static get defaults() {

        return {
            // the column is mapped to "priority" field of the Task model
            field: 'keyResult',
            text: "L{KeyResult}",
        };
    }

    get defaultEditor() {
        return {
            type: KeyResultField.type,
            // @ts-ignore
            store: this.grid.project.keyResultStore,
            valueField: 'id',
            displayField: 'description',
            clearable: true,
        }

    }

    // @ts-ignore
    renderer({value}: any) {
        if (!value) {
            return ""
        }
        // @ts-ignore
        let text = this.grid.project.keyResultStore.getById(value).description
        return text ? text : value;
    }

    onBeforeCellEditStart({editorContext}: any) {
        if (editorContext.column == this) {
            return editorContext.record.keyMilestone === true
        }
        return true
    }

    construct(config: any, store: any) {
        // @ts-ignore
        super.construct(...arguments)
        // @ts-ignore
        this.grid.on({
            beforeCellEditStart: this.onBeforeCellEditStart,
            thisObj: this
        });

    }
}

// register new column
ColumnStore.registerColumnType(KeyResultColumn);