import {Column, ColumnStore, Toast} from "@bryntum/gantt";
import TaskCostCategoryField from "src/main/typescript/components/BGantt/widget/TaskCostCategoryField";

export default class TaskCostCategoryColumn extends Column {
    static get $name() {
        return 'TaskCostCategoryColumn';
    }

    // unique alias of the column
    static get type() {
        return 'taskCostCategory';
    }

    // indicates that the column should be present in "Add New..." column
    static get isGanttColumn() {
        return true;
    }

    static get defaults() {
        return {
            // the column is mapped to "priority" field of the Task model
            field: 'taskCostCategory',
            text: "L{TaskCostCategory}",
            cellCl: "b-task-cost-category"
        };
    }

    get defaultEditor() {
        return {
            type: TaskCostCategoryField.type,
            // @ts-ignore
            store: this.grid.project.taskCostCategoryStore,
        }

    }

    // @ts-ignore
    renderer({cellElement, value, record}: any) {
        if (!record.parent.taskCostCategory) {
            cellElement.classList.remove("b-grid-cell-disabled")
        } else {
            cellElement.classList.add("b-grid-cell-disabled")
        }
        if (!value) {
            return ""
        }
        // @ts-ignore
        let text = this.grid.project.taskCostCategoryStore.getById(value).name
        return text ? text : value;
    }

    onBeforeCellEditStart({editorContext}: any) {
        if (editorContext.column == this) {
            let isPrevent = !editorContext.record.parent.taskCostCategory
            if (!isPrevent) {

                Toast.show({
                    html: this.L('Parent has cost category'),
                    showProgress: false,
                    color: "b-red"
                })
            }
            return isPrevent
        }
        return true
    }

    construct(config: any, store: any) {
        // @ts-ignore
        super.construct(...arguments)
        // @ts-ignore
        //TODO найти вариант лучше
        this.grid.on({
            beforeCellEditStart: this.onBeforeCellEditStart,
            thisObj: this
        });
    }
}

// register new column
ColumnStore.registerColumnType(TaskCostCategoryColumn);