import React, {CSSProperties, useState} from "react";
import {useTranslation} from "react-i18next";
import {Affix, Button, ButtonProps, Divider, Space} from "antd";
import {GToolBarProps} from "src/main/typescript/components/BGantt/toolbar/GToolBar";
import {FullscreenExitOutlined, MinusOutlined, PlusOutlined} from "@ant-design/icons";

import "./GToolBar.css"
import {zoomIn, zoomOut, zoomToFit, dragPlan} from "src/main/typescript/components/BGantt/toolbar/ToolbarActions";
import {HandOutlined} from "src/main/typescript/components/Icons/Icons";

const toggleButtonStyle = (isButtonEnabled: boolean, color: string = 'magenta'): CSSProperties => ({
    color: isButtonEnabled ? color : undefined
})

export const GBottomToolBar: React.FC<GToolBarProps> = ({ganttRef}) => {
    const {t} = useTranslation();
    const [isDragButtonEnabled, setIsDragButtonEnabled] = useState<boolean>(false)


    const defBtnProps = {} as ButtonProps
    const onZoomInClick = (event: any) => {
        zoomIn(ganttRef)
    }

    const onZoomOutClick = (event: any) => {
        zoomOut(ganttRef)
    }

    const onZoomToFitClick = (event: any) => {
        zoomToFit(ganttRef)
    }

    const onDragClick = (event: any) => {
        setIsDragButtonEnabled((prev: boolean) => {
            dragPlan(ganttRef, !prev)
            return !prev
        })
    }

    return (<Affix className={"sberproject-gbottomtoolbar-affix"}>
            <div className={"sberproject-gbottomtoolbar-panel"}>
                <Space size={12}>
                    <Button {...defBtnProps} key={"zoomInButton"} onClick={onZoomInClick} icon={<PlusOutlined/>}/>
                    <Button {...defBtnProps} key={"zoomOutButton"} onClick={onZoomOutClick} icon={<MinusOutlined/>}/>
                    <Button {...defBtnProps} key={"dragButton"}
                            onClick={onDragClick}
                            icon={<HandOutlined style={toggleButtonStyle(isDragButtonEnabled)}/>}/>
                    <Divider type={'vertical'}/>
                    <Button {...defBtnProps} key={"zoomToFit"} onClick={onZoomToFitClick}
                            icon={<FullscreenExitOutlined/>}/>
                </Space>
            </div>
        </Affix>
    )
}