import React, {useEffect, useRef} from "react";
import {Tabs} from 'antd';
import {ViewTab} from "src/main/typescript/components/BGantt/toolbar/tabs/ViewTab";
import {TaskTab} from "src/main/typescript/components/BGantt/toolbar/tabs/TaskTab";
import {useTranslation} from "react-i18next";
import './GToolBar.css'
import {useGToolbarHeaderButtons} from "src/main/typescript/components/BGantt/toolbar/HeaderButtonGroup";
import * as erpConst from "src/main/typescript/Utils/ErpConstants";
import {FormBaseUrl} from "src/main/typescript/Utils/ErpConstants";
import {useHistory} from "react-router";
import {savePlan, syncCurrentPlanWithWork} from "src/main/typescript/components/BGantt/toolbar/ToolbarActions";
import {GStateProvider} from "src/main/typescript/components/BGantt/toolbar/GStateProvider";
import {useApolloClient} from "@apollo/client";
import {
    SearchPlanDocument,
    SearchPlanQuery,
    SearchPlanQueryVariables
} from "src/main/typescript/__generated/graphql-types";
import {ApolloError} from "@apollo/client/errors";
import {GraphQLError} from "graphql/error/GraphQLError";


interface PlanInfo {
    id?: string,
    name?: string,
    type?: string,
    readonly?: boolean,
    lastChangeDate?: string,
}

/**
 * TODO удалить, когда будет сделан глобальный контекст https://jira.sberbank.ru/browse/RNDTEAM-442
 */
export interface AccObjectInfo {
    id?: string,
    name?: string,
    currentPlan?: PlanInfo
    workPlan?: PlanInfo
}

export interface GToolBarProps {
    isReadOnly: boolean,
    ganttRef: object,
    accObject?: AccObjectInfo
    baseUrl: FormBaseUrl
}

export interface DefaultTabProps {
    isReadOnly: boolean,
    ganttRef: any,
    objectId: string
}

export const GToolBar: React.FC<GToolBarProps> = ({isReadOnly, ganttRef, objectId, accObject, baseUrl}) => {
    const {t} = useTranslation();
    const routerHistory = useHistory();
    const client = useApolloClient();
    const lastChangeDateRef = useRef(accObject?.currentPlan?.lastChangeDate);

    const toPlanPage = (accObjectId: string) => {
        routerHistory.push(`${erpConst.publicPath}/${baseUrl}/${accObjectId}/plans`)
    };

    const onPlanListClick = (event: any) => {
        toPlanPage(accObject?.id!)
    }
    const onSaveClick = async (event: any) => {
        const getActualCurrentPlan = () => client.query<SearchPlanQuery, SearchPlanQueryVariables>({
            query: SearchPlanDocument,
            variables: {searchCond: "it.$id == '" + accObject?.currentPlan?.id + "'"},
            fetchPolicy: 'no-cache',
        })
            .then(result => result.data.searchPlan.elems[0]);
        const actualCurrentPlan = await getActualCurrentPlan();

        if (!!lastChangeDateRef.current && !!actualCurrentPlan?.lastChangeDate &&
            lastChangeDateRef.current !== actualCurrentPlan.lastChangeDate) {
            throw new ApolloError({
                graphQLErrors: [new GraphQLError('', null, null, null, null, null, {
                    classification: "COMPARE_NOT_EQUAL",
                })]
            })
        }

        const response = await savePlan(ganttRef);
        const afterSavePlanData = await getActualCurrentPlan();
        lastChangeDateRef.current = afterSavePlanData.lastChangeDate;

        return response;
    }
    const onSyncClick = (event: any): Promise<any> => {
        return syncCurrentPlanWithWork(ganttRef)
    }


    useGToolbarHeaderButtons({
        onPlanListClick: onPlanListClick,
        saveAction: onSaveClick,
        syncAction: onSyncClick,
        accObject: accObject,
        entity: baseUrl == FormBaseUrl.productVersion ? "Version" : "Project",
    }, ganttRef)

    useEffect(() => {
        lastChangeDateRef.current = accObject?.currentPlan?.lastChangeDate;
    }, [accObject?.currentPlan?.lastChangeDate]);


    return (
        <GStateProvider>
            <div className={"sberproject-gtoolbar"}>
                <Tabs defaultActiveKey="TaskTab">
                    <Tabs.TabPane tab={t("Task")} key={"taskTab"}>
                        <TaskTab objectId={accObject?.id!!} isReadOnly={isReadOnly} ganttRef={ganttRef}/>
                    </Tabs.TabPane>
                    {/*<Tabs.TabPane tab={t("Formatting")} key={"formattingTab"}>*/}
                    {/*    <FormattingTab ganttRef={ganttRef}/>*/}
                    {/*</Tabs.TabPane>*/}
                    <Tabs.TabPane tab={t("Display")} key={"viewTab"}>
                        <ViewTab objectId={accObject?.id!!} isReadOnly={isReadOnly} ganttRef={ganttRef}/>
                    </Tabs.TabPane>
                </Tabs>
            </div>
        </GStateProvider>)
}