import React, {useEffect, useState} from "react";
import {useTranslation} from "react-i18next";
import {SwitchProps} from "antd/lib/switch";
import {Switch} from "antd";
import "../GToolBar.css"
import {useGStateContext} from "src/main/typescript/components/BGantt/toolbar/GStateProvider";

interface FeatureToggleProps {
    featureName: string,
    label: string,
    ganttRef: any,
}

const FeatureToggle: React.FC<FeatureToggleProps> = ({featureName, label, ganttRef}: any) => {

    const {t} = useTranslation();
    const [featureState, setFeatureState] = useState<boolean>(false)
    const [hidden, setHidden] = useState<boolean>(false)
    const {hasGanttChanges,setHasGanttChanges} = useGStateContext()

    const defSwchConf = {size: "small"} as SwitchProps

    useEffect(() => {
        if (ganttRef) {
            if (ganttRef.features[featureName]) {
                setFeatureState(!ganttRef.features[featureName].disabled)
                setHidden(false)
            } else {
                setHidden(true)
            }
            setHasGanttChanges(false)
        }
    }, [ganttRef,hasGanttChanges])

    const onChangeFeature = (value: any) => {
        setFeatureState(value)
        if (ganttRef) {
            ganttRef.features[featureName].disabled = !value
        }
    }

    return (hidden ? null : <div className={"sberproject-gtoolbar-switcher"}>
        <Switch {...defSwchConf} style={{marginRight: "8px"}}
                checked={featureState} onChange={onChangeFeature}/>
        {t(label)}
    </div>)
}

export default FeatureToggle