import React from "react";
import {Button, Divider, Dropdown, Menu, Space} from "antd";
import {DefaultTabProps} from "../GToolBar";
import {useTranslation} from "react-i18next";
import {
    CaretRightOutlined,
    DeleteOutlined,
    EditOutlined,
    LinkOutlined,
    MenuFoldOutlined,
    MenuUnfoldOutlined,
    MoreOutlined,
    PauseOutlined,
    PlusOutlined,
    PushpinOutlined
} from '@ant-design/icons';
import {BaseButtonProps} from "antd/lib/button/button";
import '../GToolBar.css'
import {
    addNewTask,
    changePercentDoneOnSelectedTasks,
    deleteSelectedTasks,
    editSelectedTask,
    indentSelectedTasks,
    linkSelectedTasks,
    onScheduleAction,
    outdentSelectedTasks,
    selectedTasksToActive,
    selectedTasksToAutoScheduled,
    selectedTasksToInactive,
    selectedTasksToManuallyScheduled,
    unlinkSelectedTasks
} from "src/main/typescript/components/BGantt/toolbar/ToolbarActions";
import {AutoScheduleOutlined, UnlinkOutlined} from "../../../Icons/Icons";

interface TaskTabProps extends DefaultTabProps {
}

export const TaskTab: React.FC<TaskTabProps> = ({isReadOnly, ganttRef}) => {
    const {t} = useTranslation();

    const defBtnProp = {disabled: isReadOnly} as BaseButtonProps

    const onAddNewTask = (event: any) => {
        switch (event.key) {
            case "createTaskMenuItem":
                addNewTask(ganttRef, "task")
                break;
            case "createSummaryTaskMenuItem":
                addNewTask(ganttRef, "summarytask")
                break;
            case "createMilestoneMenuItem":
                addNewTask(ganttRef, "milestone")
                break;
            default:
                addNewTask(ganttRef, 'task')
                break;
        }
    }
    const onEditTask = (event: any) => {
        editSelectedTask(ganttRef)
    }
    const onDeleteTask = (event: any) => {
        deleteSelectedTasks(ganttRef)
    }

    //Один из способов передать значение при нажатии кнопки. Функция возрвщает функцию
    const onChangePercentDone = (value: number) => (event: any) => {
        changePercentDoneOnSelectedTasks(ganttRef, value)
    }
    const onManuallyScheduled = (event: any) => {
        selectedTasksToManuallyScheduled(ganttRef)
    }
    const onAutoScheduled = (event: any) => {
        selectedTasksToAutoScheduled(ganttRef)
    }

    const onLinkTask = (event: any) => {
        linkSelectedTasks(ganttRef)
    }
    const onUnlinkTask = (event: any) => {
        unlinkSelectedTasks(ganttRef)
    }

    const onIndentTask = (event: any) => {
        indentSelectedTasks(ganttRef)
    }
    const onOutdentTask = (event: any) => {
        outdentSelectedTasks(ganttRef)
    }
    const onActivateTask = (event: any) => {
        selectedTasksToActive(ganttRef)
    }
    const onDeactivateTask = (event: any) => {
        selectedTasksToInactive(ganttRef)
    }
    const onSchedule = (event: any) => {
        onScheduleAction(ganttRef)
    }

    const percentDoneDropdownMenu = (<Menu className={"sberproject-gtoolbar-dropdown"}>
        <Menu.Item disabled={isReadOnly || true} key={"keepDependenciesMenuItem"}>{t("KeepDependencies")}</Menu.Item>
        <Menu.Item disabled={isReadOnly} key={"onScheduleMenuItem"} onClick={onSchedule}>{t("OnSchedule")}</Menu.Item>
    </Menu>)

    const createTaskDropdownMenu = (
        <Menu disabled={isReadOnly} className={"sberproject-gtoolbar-dropdown"} onClick={onAddNewTask}>
            <Menu.Item key={"createTaskMenuItem"}
                       defaultValue={"Task"}>{t("Task")}</Menu.Item>
            <Menu.Item key={"createSummaryTaskMenuItem"}
                       defaultValue={"SummaryTask"}>{t("SummaryTask")}</Menu.Item>
            <Menu.Item key={"createMilestoneMenuItem"}
                       defaultValue={"Milestone"}>{t("Milestone")}</Menu.Item>
        </Menu>
    )
    const crudTaskGroup = <Button.Group className={"sberproject-gtoolbar-buttongroup"}>
        <Dropdown disabled={isReadOnly} overlay={createTaskDropdownMenu}>
            <Button {...defBtnProp} key={"addTaskButton"} icon={<PlusOutlined/>}/>
        </Dropdown>
        <Button {...defBtnProp} key={"editTaskButton"} icon={<EditOutlined/>} onClick={onEditTask}/>
        <Button {...defBtnProp} key={"deleteTaskButton"} icon={<DeleteOutlined/>} onClick={onDeleteTask}
                danger={true}/>
    </Button.Group>
    const percentDoneGroup = <Button.Group className={"sberproject-gtoolbar-buttongroup"}>
        <Button {...defBtnProp} key={"changePercentDoneButton0"} onClick={onChangePercentDone(0)}>0%</Button>
        <Button {...defBtnProp} key={"changePercentDoneButton25"} onClick={onChangePercentDone(25)}>25%</Button>
        <Button {...defBtnProp} key={"changePercentDoneButton50"} onClick={onChangePercentDone(50)}>50%</Button>
        <Button {...defBtnProp} key={"changePercentDoneButton75"} onClick={onChangePercentDone(75)}>75%</Button>
        <Dropdown.Button  {...defBtnProp} key={"changePercentDoneButton100"}
                          onClick={onChangePercentDone(100)}
                          icon={<MoreOutlined/>} overlay={percentDoneDropdownMenu}>100%</Dropdown.Button>
    </Button.Group>

    const somethingGroup = <Button.Group className={"sberproject-gtoolbar-buttongroup"}>
        <Button {...defBtnProp} key={"manualSchedulingButton"} icon={<PushpinOutlined/>}
                onClick={onManuallyScheduled}/>
        <Button {...defBtnProp} key={"autoSchedulingButton"}
                icon={<AutoScheduleOutlined onClick={onAutoScheduled}/>}/>
    </Button.Group>

    const linksGroup = <Button.Group className={"sberproject-gtoolbar-buttongroup"}>
        <Button {...defBtnProp} key={"linkTaskButton"} icon={<LinkOutlined/>} onClick={onLinkTask}/>
        <Button {...defBtnProp} key={"unlinkTaskButton"} icon={<UnlinkOutlined/>}
                onClick={onUnlinkTask}/>
    </Button.Group>

    const dentGroup = <Button.Group className={"sberproject-gtoolbar-buttongroup"}>
        <Button {...defBtnProp} key={"indentTaskButton"} icon={<MenuUnfoldOutlined/>}
                onClick={onIndentTask}/>
        <Button {...defBtnProp} key={"outdentTaskButton"} icon={<MenuFoldOutlined/>}
                onClick={onOutdentTask}/>
    </Button.Group>

    const activationGroup = <Button.Group className={"sberproject-gtoolbar-buttongroup"}>
        <Button {...defBtnProp} key={"activateTaskButton"} icon={<CaretRightOutlined/>}
                onClick={onActivateTask}/>
        <Button {...defBtnProp} key={"deactivateTaskButton"} icon={<PauseOutlined/>} onClick={onDeactivateTask}/>
    </Button.Group>
    return (<>
        <Space>
            <Space size={24}>
                {crudTaskGroup}
                {percentDoneGroup}
                {somethingGroup}
                {linksGroup}
                {dentGroup}
            </Space>
            <Divider type={"vertical"}/>
            <Space size={24}>
                {activationGroup}
            </Space>
        </Space>
    </>)
}