import React, {useEffect, useState} from "react";
import {Badge, Button, Divider, Slider, Space} from "antd";
import {DefaultTabProps} from "../GToolBar";
import {useTranslation} from "react-i18next";
import {ViewStateSelector} from "src/main/typescript/components/BGantt/toolbar/ViewStateSelector";
import {ClearOutlined} from "@ant-design/icons";
import "../GToolBar.css"
import FeatureToggle from "src/main/typescript/components/BGantt/toolbar/tabs/FeatureToggle";
import {
    clearTaskStoreFilters,
    clearTaskStoreSorters
} from "src/main/typescript/components/BGantt/toolbar/ToolbarActions";
import {useGStateContext} from "src/main/typescript/components/BGantt/toolbar/GStateProvider";

interface ViewTabProps extends DefaultTabProps {
}


export const ViewTab: React.FC<ViewTabProps> = ({ganttRef, objectId}) => {
    const {t} = useTranslation();

    const [sortCriteriaCount, setSortCriteriaCount] = useState<number>()
    const [filterCriteriaCount, setFilterCriteriaCount] = useState<number>()
    const [rowHeightSliderValue, setRowHeightSliderValue] = useState<number>(20)
    const [barMarginSliderValue, setBarMarginSliderValue] = useState<number>()
    const {hasGanttChanges} = useGStateContext()


    const triggerFilterEvent = (event: { filters: any }) => {
        setFilterCriteriaCount(event.filters.count)
    }
    const triggerSortEvent = (event: { sorters: any }) => {
        setSortCriteriaCount(event.sorters.length)
    }

    useEffect(() => {

        return () => {
            if (ganttRef) {
                ganttRef?.taskStore?.un({
                    filter: triggerFilterEvent,
                    sort: triggerSortEvent,
                    beforeSort: triggerSortEvent
                })
            }
        }
    }, [])


    useEffect(() => {
        if (ganttRef) {
            ganttRef.taskStore.on({
                filter: triggerFilterEvent,
                beforeSort: triggerSortEvent
            });
        }
    }, [ganttRef])

    useEffect(() => {
        if (ganttRef) {
            setRowHeightSliderValue(ganttRef.rowHeight)
            setBarMarginSliderValue(ganttRef.barMargin)
            setFilterCriteriaCount(ganttRef?.taskStore.filters.count)
            setSortCriteriaCount(ganttRef?.taskStore?.sorters?.length)
        }
    }, [hasGanttChanges])

    const onClearSorting = (event: any) => {
        clearTaskStoreSorters(ganttRef)
    }

    const onClearFiltering = (event: any) => {
        clearTaskStoreFilters(ganttRef)
    }

    const onChangeRowHeight = (value: any) => {
        setRowHeightSliderValue(value)
        if (ganttRef) {
            ganttRef.rowHeight = value;
        }
    }
    const onBarMargin = (value: any) => {
        setBarMarginSliderValue(value)
        if (ganttRef) {
            ganttRef.barMargin = value
        }
    }
    const sortingGroup = <Space size={4} className={"sberproject-gtoolbar-sorts-group"}>
        {t("Sorting")}
        <Badge showZero={true} count={sortCriteriaCount}/>
        <Button icon={<ClearOutlined/>} disabled={!sortCriteriaCount} size={"small"} onClick={onClearSorting}/>
    </Space>
    const filteringGroup = <Space size={4} className={"sberproject-gtoolbar-filters-group"}>
        {t("Filters")}
        <Badge showZero={true} count={filterCriteriaCount}/>
        <Button icon={<ClearOutlined/>} disabled={!filterCriteriaCount} size={"small"} onClick={onClearFiltering}/>
    </Space>


    const dependenciesSwitch = <FeatureToggle featureName={"dependencies"} label={"Dependencies"}
                                              ganttRef={ganttRef}/>
    const tasksLabelSwitch = <FeatureToggle featureName={"labels"} label={"Task's Label"}
                                            ganttRef={ganttRef}/>
    const criticalPathSwitch = <FeatureToggle featureName={"criticalPaths"} label={"Critical path"}
                                              ganttRef={ganttRef}/>


    const rowHeightSlider = <>
        {t("RowHeight")}
        <Slider style={{minWidth: '140px'}}
                min={20}
                max={70}
                value={rowHeightSliderValue}
                onChange={onChangeRowHeight}/>
    </>
    const edgeWidthSlider = <>
        {t("EdgeWidth")}
        <Slider style={{minWidth: '140px'}}
                min={0}
                max={(rowHeightSliderValue / 2) - 5}
                value={barMarginSliderValue}
                onChange={onBarMargin}
        />
    </>

    return (<>
        <Space>
            <Space size={24}>
                <ViewStateSelector objectId={objectId} ganttRef={ganttRef}/>
                {sortingGroup}
                {filteringGroup}
            </Space>
            <Divider type={"vertical"}/>
            <Space size={14}>
                {dependenciesSwitch}
                {tasksLabelSwitch}
                {criticalPathSwitch}
            </Space>
            <Divider type={"vertical"}/>
            <Space>
                {rowHeightSlider}
                {edgeWidthSlider}
            </Space>
        </Space>
    </>)
}

